﻿; ResizeBrowser.ahk[UTF-8] (2024/02/21 03:00)
; for AutoHotKey v2.0.12.0 or later
;
; 何故か位置やサイズが微妙に変わっていってしまうブラウザが前面に来た時に位置と
; サイズを規定値に戻します。

/* 変数初期化 */
Path := Array()
PosSize := Array()
Out := Array()
IniFile := "ResizeBrowser.ini"
IniSect := "APP"

;左上座標とサイズを格納する配列の添字
POSX := 1
POSY := 2
SIZW := 3
SIZH := 4
;四隅座標の値を格納する配列の添字
LTX := 5
LTY := 6
RTX := 7
RTY := 8
LBX := 9
LBY := 10
RBX := 11
RBY := 12
;配列の要素数を伸張
Out.Length := 12

;iniファイルから件数を読み込む
count := IniRead(IniFile, IniSect, "MAX", 0)
if( count = 0 ){
	MsgBox("iniファイルに定義がありません")
	ExitApp()
}
MoveWait := Number(IniRead(IniFile, IniSect, "MoveWait", "1000"))
NextWait := Number(IniRead(IniFile, IniSect, "NextWait", "10"))
LoopWait := Number(IniRead(IniFile, IniSect, "LoopWait", "100"))
MoveLimit := Number(IniRead(IniFile, IniSect, "MoveLimit", "100"))
idx := 1
while( count > 0 ){
	;フルパスの読み込み(PATH1～)
	strTemp := IniRead(IniFile, IniSect, "PATH" String(idx), "")
	Path.Push( strTemp )
	;座標とサイズを読み込み(WIND1～)
	strTemp := IniRead(IniFile, IniSect, "WIND" String(idx), "")
	;両端のダブルクォートを削除し、カンマ区切りを配列に分割
	strTemp := StrReplace(strTemp, '"', "")
	aryTemp := StrSplit(strTemp, ",")
	;座標とサイズを数値化
	aryTemp[1] := Number(aryTemp[1])
	aryTemp[2] := Number(aryTemp[2])
	aryTemp[3] := Number(aryTemp[3])
	aryTemp[4] := Number(aryTemp[4])
	;左上
	aryTemp.Push(aryTemp[1])
	aryTemp.Push(aryTemp[2])
	;右上
	aryTemp.Push(aryTemp[1] + aryTemp[3])
	aryTemp.Push(aryTemp[2])
	;左下
	aryTemp.Push(aryTemp[1])
	aryTemp.Push(aryTemp[2] + aryTemp[4])
	;右下
	aryTemp.Push(aryTemp[1] + aryTemp[3])
	aryTemp.Push(aryTemp[2] + aryTemp[4])

	PosSize.Push( aryTemp )

	count := count - 1
	idx := idx + 1
}

idx := 1
WS_SIZEBOX := 0x40000    ;サイズ可変ウィンドウである
WS_POPUP := 0x80000000   ;ポップアップウィンドウである

/* 順次アクティブウィンドウかどうかを切り替えて位置補正 */
Loop
{
	while( Path.Length >= idx ){
		hWnd := WinActive( "ahk_exe " Path[idx] )
		if( hWnd != 0 ){
			;//最大化状態と最小化状態では動作させない
			MinMax := -1
			try{
				MinMax := WinGetMinMax( "ahk_id " hWnd )
			} catch Error as err {
				hWnd := 0
			}
			;//マウスのドラッグ操作中やスクロール中は動作させない
			IsDown1 := GetKeyState("LButton") or GetKeyState("RButton") or GetKeyState("MButton")
			IsDown2 := GetKeyState("WheelUp") or GetKeyState("WheelDown") or GetKeyState("WheelLeft") or GetKeyState("WheelRight")
			if( MinMax = 0 and IsDown1 = 0 and IsDown2 = 0 and hWnd != 0 ){
				;//現在のウィンドウ位置とサイズを獲得し、閾値未満のずれがあったら定位置定サイズに戻す
				try{
					WinGetPos( &OutX, &OutY, &OutW, &OutH, "ahk_id " hWnd )
				} catch Error as err {
					hWnd := 0
					OutX := 0
					OutY := 0
					OutW := 0
					OutH := 0
				}
				Out[POSX] := OutX
				Out[POSY] := OutY
				Out[SIZW] := OutW
				Out[SIZH] := OutH
				;//起点座標とサイズから四隅座標を求める
				Out[LTX] := Out[POSX]
				Out[LTY] := Out[POSY]
				Out[RTX] := Out[POSX] + Out[SIZW]
				Out[RTY] := Out[POSY]
				Out[LBX] := Out[POSX]
				Out[LBY] := Out[POSY] + Out[SIZH]
				Out[RBX] := Out[POSX] + Out[SIZW]
				Out[RBY] := Out[POSY] + Out[SIZH]
				
				;//全座標が閾値以内にあり、どれか一致していない場合のみ対象とする
				if( Abs(Out[LTX] - PosSize[idx][LTX]) < MoveLimit and
					Abs(Out[LTY] - PosSize[idx][LTY]) < MoveLimit and
					Abs(Out[RTX] - PosSize[idx][RTX]) < MoveLimit and
					Abs(Out[RTY] - PosSize[idx][RTY]) < MoveLimit and
					Abs(Out[LBX] - PosSize[idx][LBX]) < MoveLimit and
					Abs(Out[LBY] - PosSize[idx][LBY]) < MoveLimit and
					Abs(Out[RBX] - PosSize[idx][RBX]) < MoveLimit and
					Abs(Out[RBY] - PosSize[idx][RBY]) < MoveLimit and
					not ( Out[LTX] = PosSize[idx][LTX] and Out[LTY] = PosSize[idx][LTY] and
						  Out[RTX] = PosSize[idx][RTX] and Out[RTY] = PosSize[idx][RTY] and
						  Out[LBX] = PosSize[idx][LBX] and Out[LBY] = PosSize[idx][LBY] and
						  Out[RBX] = PosSize[idx][RBX] and Out[RBY] = PosSize[idx][RBY] ) and hWnd != 0 ){
					
					;//サイズ変更可能なウィンドウでポップアップでない場合のみ移動リサイズする
					Style := 0
					try{
						Style := WinGetStyle( "ahk_id " hWnd )
					} catch Error as err {
						hWnd := 0
					}
					if( Style & WS_SIZEBOX and not( Style & WS_POPUP) and hWnd != 0 ){
						;MsgBox( Format( "0x{1:x}", Style ) )
						try{
							WinMove( PosSize[idx][1], PosSize[idx][2], PosSize[idx][3], PosSize[idx][4], "ahk_id " hWnd )
						} catch Error as err {
							;
						}
						Sleep(MoveWait)
					}
				}
			}
		}
		idx := idx + 1
		Sleep(NextWait)
	}
	idx := 1
	Sleep(LoopWait)
}

